@@ -335,7 +335,6 @@ void basic_properties_to_PyDict(amqp_basic_properties_t *props,
     PyObject *key = NULL;
     PyObject *value = NULL;
     PyObject *h = NULL;
-    h = PyDict_New();
 
     if (props->_flags & AMQP_BASIC_CONTENT_TYPE_FLAG) {
         PyDICT_SETSTR_DECREF(p, "content_type", value,
@@ -624,8 +623,6 @@ int PyRabbitMQ_recv(PyObject *p, amqp_connection_state_t conn, int piggyback) {
         PyDICT_SETSTR_DECREF(p, "channel", value,
                 PyInt_FromLong((long)frame.channel));
 
-        propdict = PyDict_New();
-
         // p["properties"] = {}
         props = (amqp_basic_properties_t *)frame.payload.properties.decoded;
         PyDICT_SETSTR_DECREF(p, "properties", propdict,
@@ -635,7 +632,7 @@ int PyRabbitMQ_recv(PyObject *p, amqp_connection_state_t conn, int piggyback) {
         // p["body"]
         body_target = frame.payload.properties.body_size;
         body_received = 0;
-        payload = PyString_FromStringAndSize("", 0);
+        PyObject *parts = PyList_New(0);
         while (body_received < body_target) {
             PyObject *part;
             Py_BEGIN_ALLOW_THREADS;
@@ -651,13 +648,17 @@ int PyRabbitMQ_recv(PyObject *p, amqp_connection_state_t conn, int piggyback) {
 
             body_received += frame.payload.body_fragment.len;
             part = PySTRING_FROM_AMQBYTES(frame.payload.body_fragment);
-            PyString_Concat(&payload, part);
+            if (PyList_Append(parts, part) == -1)
+                return -1;
             Py_DECREF(part);
-            if (payload == NULL) return -1;
         }
 
+        PyObject *sep = PyString_FromString("");
+        payload = _PyString_Join(sep, parts);
         PyDict_SetItemString(p, "body", payload);
         Py_DECREF(payload);
+        Py_DECREF(sep);
+        Py_DECREF(parts);
         break;
     }
     amqp_maybe_release_buffers(conn);
